#define UART_USR0 ((volatile unsigned char*)(0x40000204)) /* UART status registers / clock select */
#define UART_URB0 ((volatile unsigned char*)(0x0000020C)) /* UART receiver buffer / transmitter buffer */

#define USR_RxRDY	0x01			/* receiver ready */
#define QUEUE_SIZE      16

extern volatile char uart_queue[];	// pointer to the character queue
extern volatile unsigned uart_fetchptr;	// queue fetch pointer
extern volatile unsigned uart_storeptr;	// queue store pointer

// SCI interrupt handler
void uart_handler() __attribute__((interrupt_handler));
void uart_handler()
{
    unsigned ptrdiff;		// fetch-store pointer difference

    while( (*UART_USR0 & USR_RxRDY) )
    {

    ptrdiff = uart_fetchptr - uart_storeptr + QUEUE_SIZE;
    ptrdiff %= QUEUE_SIZE;
    if( ptrdiff == 1 )		// if the queue is full
    {
	ptrdiff = *UART_URB0;	// perform dummy read operation	
	return;			// and abort
    }
	
    uart_queue[uart_storeptr++] = *UART_URB0;	// store the character in the queue
    uart_storeptr %= QUEUE_SIZE;	// wrap around the store pointer if necessary
				// mark that the character has been fetched
				// from the receiver data register
    }
}
